/** Copyright 2014 SSC and 2002-2014 CS Systèmes d'Information
 * Licensed to CS SystÃ¨mes d'Information (CS) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * CS licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// this file was created by SCC and is largely a derived work from the
// original file NodeDetector.java created by CS Systèmes d'Information

package org.orekit.python;

import org.orekit.frames.Frame;
import org.orekit.orbits.Orbit;
import org.orekit.propagation.events.EventDetector;
import org.orekit.propagation.events.NodeDetector;
/*import org.orekit.propagation.events.handlers.EventHandler;*/
import org.orekit.propagation.SpacecraftState;
import org.orekit.errors.OrekitException;

/** Finder for node crossing events.
 * <p>This class finds equator crossing events (i.e. ascending
 * or descending node crossing).</p>
 * <p>The default implementation behavior is to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#CONTINUE continue}
 * propagation at descending node crossing and to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#STOP stop} propagation
 * at ascending node crossing. This can be changed by calling
 * {@link #withHandler(EventHandler)} after construction.</p>
 * <p>Beware that node detection will fail for almost equatorial orbits. If
 * for example a node detector is used to trigger an {@link
 * org.orekit.forces.maneuvers.ImpulseManeuver ImpulseManeuver} and the maneuver
 * turn the orbit plane to equator, then the detector may completely fail just
 * after the maneuver has been performed! This is a real case that has been
 * encountered during validation ...</p>
 * @see org.orekit.propagation.Propagator#addEventDetector(EventDetector)
 * @author Luc Maisonobe
 */
public class PythonNodeDetector extends NodeDetector
{

	private static final long serialVersionUID = 2369785604202125622L;

    /** Part of JCC Python interface to object */
	private long pythonObject;

    /** Part of JCC Python interface to object */
	public void pythonExtension(long pythonObject)
	{
		this.pythonObject = pythonObject;
	}

    /** Part of JCC Python interface to object */
	public long pythonExtension()
	{
		return this.pythonObject;
	}

    /** Part of JCC Python interface to object */
	public void finalize()
			throws Throwable
			{
		pythonDecRef();
			}

    /** Part of JCC Python interface to object */
	public native void pythonDecRef();


    /** Build a new instance.
     * <p>The orbit is used only to set an upper bound for the max check interval
     * to period/3 and to set the convergence threshold according to orbit size.</p>
     * @param orbit initial orbit
     * @param frame frame in which the equator is defined (typical
     * values are {@link org.orekit.frames.FramesFactory#getEME2000() J<sub>2000</sub>} or
     * {@link org.orekit.frames.FramesFactory#getITRF2005() ITRF 2005})
     */
	public PythonNodeDetector(final Orbit orbit, final Frame frame) {
		super(orbit, frame);
	}

    /** Build a new instance.
     * <p>The orbit is used only to set an upper bound for the max check interval
     * to period/3.</p>
     * @param threshold convergence threshold (s)
     * @param orbit initial orbit
     * @param frame frame in which the equator is defined (typical
     * values are {@link org.orekit.frames.FramesFactory#getEME2000() J<sub>2000</sub>} or
     * {@link org.orekit.frames.FramesFactory#getITRF2005() ITRF 2005})
     */
	public PythonNodeDetector(final double threshold, final Orbit orbit, final Frame frame)
	{
		super(threshold, orbit, frame);
	}

	public native Action eventOccurred(final SpacecraftState s, final boolean increasing) throws OrekitException;


}

